@ECHO OFF

REM Prompt to program a microcontroller. Programs or quits based on user input
:PROMPT
	SET /P answer=Program (S)ystem controller, program (L)aser controller, or (Q)uit (S\L\Q)? 
	IF /I "%answer:~,1%" EQU "S" GOTO SYSCTRL
	IF /I "%answer:~,1%" EQU "Q" GOTO QUIT
	IF /I "%answer:~,1%" EQU "L" GOTO LASER
	ECHO Please type P for Program Q for Quit, or L to program laser controllers
	GOTO PROMPT

REM Programs the system controller. Either reports success or tries to determine what caused the error
:SYSCTRL
	SET micro=ATMEGA64
	ECHO Programming...
	AVRDUDE.EXE -c avrisp2 -P usb -p atmega64 -y -u -U flash:w:sysctrl.hex:i > flash_error 2>&1
	TIMEOUT /T 3 > NUL
	AVRDUDE.EXE -c avrisp2 -P usb -p atmega64 -y -u -U hfuse:w:0xd9:m > fuse_error 2>&1
	C:\Windows\System32\find.exe /I "flash written" flash_error > NUL
	IF ERRORLEVEL 1 GOTO ERROR_CATCH
	C:\Windows\System32\find.exe /I "hfuse written" fuse_error > NUL
	IF ERRORLEVEL 1 GOTO ERROR_CATCH
	GOTO SC_SUCCESS

REM Programs the laser controller. Either reports success or tries to determine what caused the error
:LASER
	SET micro=ATMEGA164A
	ECHO Programming...
	AVRDUDE.EXE -c avrisp2 -P usb -p atmega164a -y -u -U flash:w:laser.hex:i > flash_error 2>&1
	TIMEOUT /T 3 > NUL
	AVRDUDE.EXE -c avrisp2 -P usb -p atmega164a -y -u -U hfuse:w:0xd1:m > fuse_error 2>&1
	C:\Windows\System32\find.exe /I "flash written" flash_error > NUL
	IF ERRORLEVEL 1 GOTO ERROR_CATCH
	C:\Windows\System32\find.exe /I "hfuse written" fuse_error > NUL
	IF ERRORLEVEL 1 GOTO ERROR_CATCH
	GOTO LASER_SUCCESS

REM checks the output of avrdude to try to determine the source of the error then returns to the prompt
:ERROR_CATCH
	ECHO Programming Failed
	C:\Windows\System32\find.exe /I "failed" flash_error > NUL
	IF ERRORLEVEL 1 (ECHO NULL > NUL) ELSE (ECHO Could not find microcontroller. Check power and programming cable)
	C:\Windows\System32\find.exe /I "Expected signature" flash_error > NUL
	IF ERRORLEVEL 1 (ECHO NULL > NUL) ELSE (ECHO Wrong microcontroller. Expected %micro%.)
	C:\Windows\System32\find.exe /I "did not find any USB" flash_error > NUL
	IF ERRORLEVEL 1 (ECHO NULL > NUL) ELSE (ECHO Programming cable not found. Check cable or reinstall drivers.)
	C:\Windows\System32\find.exe /I "did not find any USB" fuse_error > NUL
	IF ERRORLEVEL 1 (ECHO NULL > NUL) ELSE (ECHO Fuse programming failed. Reprogramming may fix this.)
	GOTO PROMPT

REM Reports that the system controller was programmed successfully and then returns to the prompt
:SC_SUCCESS
	ECHO System controller programming successful.
	GOTO PROMPT

REM Reports that the laser controller was programmed successfully and then returns to the prompt
:LASER_SUCCESS
	ECHO Laser controller programming successful.
	GOTO PROMPT

REM Exits the script
:QUIT
	echo Finished Programming. Thank you.
